﻿using System;

using Android.App;
using Android.Content.PM;
using Android.Runtime;
using Android.OS;
using Com.Scansolutions.Mrzscannerlib;
using System.IO;
using Android.Graphics;

namespace MRZScannerForms.Droid
{
    [Activity(Label = "MRZScannerForms", Icon = "@mipmap/icon", Theme = "@style/MainTheme", MainLauncher = true, ConfigurationChanges = ConfigChanges.ScreenSize | ConfigChanges.Orientation | ConfigChanges.UiMode | ConfigChanges.ScreenLayout | ConfigChanges.SmallestScreenSize )]
    public class MainActivity : global::Xamarin.Forms.Platform.Android.FormsAppCompatActivity,
        IMRZScannerListener, IMRZLicenceResultListener
    {
        public static ScannerControl elementScanner;

        protected override void OnCreate(Bundle savedInstanceState)
        {
            base.OnCreate(savedInstanceState);

            Xamarin.Essentials.Platform.Init(this, savedInstanceState);
            global::Xamarin.Forms.Forms.Init(this, savedInstanceState);
            LoadApplication(new App());
        }
        public override void OnRequestPermissionsResult(int requestCode, string[] permissions, [GeneratedEnum] Android.Content.PM.Permission[] grantResults)
        {
            Xamarin.Essentials.Platform.OnRequestPermissionsResult(requestCode, permissions, grantResults);

            base.OnRequestPermissionsResult(requestCode, permissions, grantResults);
        }

        public void OnRegisterWithLicenceResult(int result, string errorMessage)
        {
            LicenseResultType licenseResult = LicenseResultType.Unknown;

            switch (result)
            {
                case 0:
                    licenseResult = LicenseResultType.Successful;
                    break;
                case -1:
                    licenseResult = LicenseResultType.Error_While_Parsing;
                    break;
                case -2:
                    licenseResult = LicenseResultType.Invalid_Licence;
                    break;
                case -3:
                    licenseResult = LicenseResultType.Invalid_Bundle_ID;
                    break;
                case -4:
                    licenseResult = LicenseResultType.Invalid_Device_Model;
                    break;
                case -5:
                    licenseResult = LicenseResultType.Licence_Expired;
                    break;
                case -6:
                    licenseResult = LicenseResultType.Invalid_Platform;
                    break;
            }

            elementScanner.OnLicenseResult(licenseResult);
        }

        public void SuccessfulScanWithResult(MRZResultModel mrzResultModel)
        {
            if (elementScanner != null)
            {
                ResultModel _resultReceived = new ResultModel(mrzResultModel.DocumentTypeRaw, mrzResultModel.IssuingCountry, mrzResultModel.MasterCheckDigit, mrzResultModel.Surnames,
                mrzResultModel.GivenNames, mrzResultModel.DocumentNumber, mrzResultModel.DocumentNumberWithCheckDigit, mrzResultModel.Nationality, mrzResultModel.DobRaw, mrzResultModel.Sex,
                mrzResultModel.EstIssuingDateRaw, mrzResultModel.ExpirationDateRaw, mrzResultModel.Optionals, mrzResultModel.DateScanned, mrzResultModel.IsExpired,
                mrzResultModel.EstIssuingDateReadable, mrzResultModel.ExpirationDateReadable, mrzResultModel.ExpirationDateWithCheckDigit, mrzResultModel.DocumentTypeReadable, mrzResultModel.DobReadable, mrzResultModel.DobWithCheckDigit,
                mrzResultModel.FullName, mrzResultModel.RawResult);

                if (mrzResultModel.Portrait != null)
                {
                    using (MemoryStream stream = new MemoryStream())
                    {
                        mrzResultModel.Portrait.Compress(Bitmap.CompressFormat.Png, 0, stream);
                        _resultReceived.Portrait = stream.ToArray();
                    }
                }

                if (mrzResultModel.Signature != null)
                {
                    using (MemoryStream stream = new MemoryStream())
                    {
                        mrzResultModel.Signature.Compress(Bitmap.CompressFormat.Png, 0, stream);
                        _resultReceived.Signature = stream.ToArray();
                    }
                }

                if (mrzResultModel.FullImage != null)
                {
                    using (MemoryStream stream = new MemoryStream())
                    {
                        mrzResultModel.FullImage.Compress(Bitmap.CompressFormat.Png, 0, stream);
                        _resultReceived.FullImage = stream.ToArray();
                    }
                }

                if (mrzResultModel.IdFront != null)
                {
                    using (MemoryStream stream = new MemoryStream())
                    {
                        mrzResultModel.IdFront.Compress(Bitmap.CompressFormat.Png, 0, stream);
                        _resultReceived.IdFront = stream.ToArray();
                    }
                }

                if (mrzResultModel.IdBack != null)
                {
                    using (MemoryStream stream = new MemoryStream())
                    {
                        mrzResultModel.IdBack.Compress(Bitmap.CompressFormat.Png, 0, stream);
                        _resultReceived.IdBack = stream.ToArray();
                    }
                }

                elementScanner.OnScanningFinished(_resultReceived);
            }
        }

        public void SuccessfulScanWithDocumentImage(Bitmap bitmap)
        {
            if (elementScanner != null)
            {
                ResultModel resultWithImage = new ResultModel();

                if (bitmap != null)
                {
                    using (MemoryStream stream = new MemoryStream())
                    {
                        bitmap.Compress(Bitmap.CompressFormat.Png, 0, stream);
                        resultWithImage.ResultImage = stream.ToArray();
                    }
                }

                elementScanner.OnScanningFinished(resultWithImage);
            }
        }

        public void ScanImageFailed()
        {
        }

        public void PermissionsWereDenied()
        {
            if (elementScanner != null)
                elementScanner.OnScanningFinished(new ResultModel("Permissions were denied"));
        }

        public void SuccessfulIdFrontImageScan(Bitmap fullImage, Bitmap portrait)
        {
            if (elementScanner != null)
            {
                ResultModel resultWithImage = new ResultModel();

                if (fullImage != null)
                {
                    using (MemoryStream stream = new MemoryStream())
                    {
                        fullImage.Compress(Bitmap.CompressFormat.Png, 0, stream);
                        resultWithImage.IdFront = stream.ToArray();
                    }
                }

                if (portrait != null)
                {
                    using (MemoryStream stream = new MemoryStream())
                    {
                        portrait.Compress(Bitmap.CompressFormat.Png, 0, stream);
                        resultWithImage.Portrait = stream.ToArray();
                    }
                }

                elementScanner.OnScanningFinished(resultWithImage);
            }
        }
    }
}
